using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace PolygonTest
{
    public class Polygon
    {
        private int m_X1, m_Y1, m_X2, m_Y2, m_X3, m_Y3, m_X4, m_Y4;

        private Point[] m_Points = null;
        private Point m_Center = Point.Empty;

        public int X1 { get { return m_X1; } set { m_X1 = value; RebuildPoints(); } }
        public int Y1 { get { return m_Y1; } set { m_Y1 = value; RebuildPoints(); } }
        public int X2 { get { return m_X2; } set { m_X2 = value; RebuildPoints(); } }
        public int Y2 { get { return m_Y2; } set { m_Y2 = value; RebuildPoints(); } }
        public int X3 { get { return m_X3; } set { m_X3 = value; RebuildPoints(); } }
        public int Y3 { get { return m_Y3; } set { m_Y3 = value; RebuildPoints(); } }
        public int X4 { get { return m_X4; } set { m_X4 = value; RebuildPoints(); } }
        public int Y4 { get { return m_Y4; } set { m_Y4 = value; RebuildPoints(); } }
        public Point[] Points { get { return m_Points; } }
        public Point Center { get { return m_Center; } }

        public int MinX { get { return Math.Min(X1, Math.Min(X2, Math.Min(X3, X4))); } }
        public int MaxX { get { return Math.Max(X1, Math.Max(X2, Math.Max(X3, X4))); } }
        public int MinY { get { return Math.Min(Y1, Math.Min(Y2, Math.Min(Y3, Y4))); } }
        public int MaxY { get { return Math.Max(Y1, Math.Max(Y2, Math.Max(Y3, Y4))); } }

        public Polygon(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
        {
            m_X1 = x1;
            m_Y1 = y1;
            m_X2 = x2;
            m_Y2 = y2;
            m_X3 = x3;
            m_Y3 = y3;
            m_X4 = x4;
            m_Y4 = y4;
            RebuildPoints();
        }

        private void RebuildPoints()
        {
            m_Points = new Point[] { new Point(m_X1, m_Y1), new Point(m_X2, m_Y2), new Point(m_X3, m_Y3), new Point(m_X4, m_Y4) };
            m_Center = PolygonCenter(m_Points);
        }

        public bool ContainsPoint(int x, int y)
        {
            return PointInPolygon(m_Points, x, y);
        }


        /// <summary>
        /// Compute the dot product AB  BC
        /// </summary>
        private static int PointDotProduct(Point A, Point B, Point C)
        {
            Point AB = new Point();
            Point BC = new Point();
            AB.X = B.X - A.X;
            AB.Y = B.Y - A.Y;
            BC.X = C.X - B.X;
            BC.Y = C.Y - B.Y;
            int dot = AB.X * BC.X + AB.Y * BC.Y;
            return dot;
        }
        /// <summary>
        /// Compute the cross product AB x AC
        /// </summary>
        private static int PointCrossProduct(Point A, Point B, Point C)
        {
            Point AB = new Point();
            Point AC = new Point();
            AB.X = B.X - A.X;
            AB.Y = B.Y - A.Y;
            AC.X = C.X - A.X;
            AC.Y = C.Y - A.Y;
            int cross = AB.X * AC.Y - AB.Y * AC.X;
            return cross;
        }
        /// <summary>
        /// Compute the distance from A to B
        /// </summary>
        public static double PointDistance(Point A, Point B)
        {
            int d1 = A.X - B.X;
            int d2 = A.Y - B.Y;
            return Math.Sqrt(d1 * d1 + d2 * d2);
        }
        /// <summary>
        /// Compute the distance from AB to C
        /// if isSegment is true, AB is a segment, not a line.
        /// </summary>
        public static double LinePointDistance(Point A, Point B, Point C, Boolean isSegment)
        {
            if((object)A == null || (object)B == null || (object)C == null) return 0;
            if(isSegment) {
                int dot1 = PointDotProduct(A, B, C);
                if(dot1 > 0) return PointDistance(B, C);
                int dot2 = PointDotProduct(B, A, C);
                if(dot2 > 0) return PointDistance(A, C);
            }
            double line_length = PointDistance(A, B);
            if(line_length == 0)
                return Math.Abs(PointDistance(A, C));
            else
                return Math.Abs(PointCrossProduct(A, B, C) / line_length);
        }

        /// <summary>
        /// The function will return TRUE if the point is inside the
        /// polygon, or FALSE if it is not. If the point is exactly on
        /// the edge of the polygon, then the function may return TRUE or FALSE.
        /// </summary>
        public static Boolean PointInPolygon(Point[] points, int X, int Y)
        {
            if(points == null) return false;
            Boolean inside = false;
            for(int i = 0; i < points.Length; i++) {
                int j = i + 1; if(j == points.Length) j = 0;
                if((points[i].Y <= Y && points[j].Y > Y)
                || (points[i].Y > Y && points[j].Y <= Y))
                    if(X < points[i].X + ((float)(Y - points[i].Y) / (points[j].Y - points[i].Y)) * (points[j].X - points[i].X))
                        inside = !inside;
            }
            return inside;
        }


        /// <summary>
        /// Returns center of polygon.
        /// </summary>
        public static Point PolygonCenter(Point[] points)
        {
            if(points != null && points.Length == 4) {
                return new Point(
                    (points[0].X + points[1].X + points[2].X + points[3].X) / 4,
                    (points[0].Y + points[1].Y + points[2].Y + points[3].Y) / 4);
            }
            return Point.Empty;
        }


        /// <summary>
        /// Returns smallest rectangle that contains all given points
        /// </summary>
        public static Rectangle RectangleAroundPoints(Point[] points)
        {
            if(points == null) return new Rectangle();
            Rectangle rc = new Rectangle(0, 0, 0, 0);
            if(points.Length >= 1) {
                rc.X = points[0].X;
                rc.Y = points[0].Y;
            }
            foreach(Point p in points) {
                if(p.X < rc.Left) { rc.Width += (rc.Left - p.X); rc.X = p.X; }
                if(p.Y < rc.Top) { rc.Height += (rc.Top - p.Y); rc.Y = p.Y; }
                if(p.X > (rc.X + rc.Width)) rc.Width = p.X - rc.X;
                if(p.Y > (rc.Y + rc.Height)) rc.Height = p.Y - rc.Y;
            }
            return rc;
        }

        /// <summary>
        /// Creates rectangle around point (p.x - size, p.y - size)(size * 2, size * 2)
        /// </summary>
        public static Rectangle RectangleAroundPoint(Point p, int size)
        {
            return new Rectangle(p.X - size, p.Y - size, size * 2, size * 2);
        }




        public static int NextPolygon(List<Polygon> polygons, int current, Keys direction)
        {
            if(polygons != null && polygons.Count > 0 && current >= 0 && current < polygons.Count) {
                Point currentCenter = polygons[current].Center;
                int min_index = -1;
                double min_dist = -1;
                for(int i = 0; i < polygons.Count; i++) {
                    if((direction == Keys.Up && polygons[i].Center.Y < (currentCenter.Y - 4)) ||
                       (direction == Keys.Down && polygons[i].Center.Y > (currentCenter.Y + 4)) ||
                       (direction == Keys.Left && polygons[i].Center.X < (currentCenter.X - 4)) ||
                       (direction == Keys.Right && polygons[i].Center.X > (currentCenter.X + 4))) {
                        double dist = PointDistance(currentCenter, polygons[i].Center);
                        if(min_dist < 0 || dist < min_dist) { min_dist = dist; min_index = i; }
                    }
                }
                return min_index;
            }
            return -1;
        }
    }
}
